#include <ESP8266WiFi.h>
#include <Servo.h>
#include <ThingSpeak.h>

Servo servo;

const char* ssid = "Your SSID"; //Your WiFi Name/SSID
const char* password = "Your Password"; //Your WiFi Password

int val;
 
int LM35    = A0;        // ADC0---A0 of NodeMCU
int FAN     = 15;        // GPIO15---D8 of NodeMCU
int ledPin  = 12;        // GPIO12---D6 of NodeMCU
int ledPin2 = 14;        // GPIO14---D5 of NodeMCU
int PIR     = 13;        // GPIO13---D7 of NodeMCU

WiFiServer server(80);
WiFiClient client;

unsigned long myChannelNumber = ThinkSpeak Channel ID; //ThinkSpeak Channel Number
const char* myWriteAPIKey = "Write API Key"; //ThingSpeak WRITE API Key
 
void setup ()
{ 
  Serial.begin(115200);
  delay(10);

  servo.attach(2);                // GPIO02 of NodeMCU with PWM pin of servo motor

  pinMode (PIR,INPUT);            // declare PIR Sensor as input
  
  pinMode (LM35,INPUT);           // declare LM35 Sensor as input
  pinMode (FAN,OUTPUT);           // declare Fan as output
  
  pinMode (ledPin, OUTPUT);       // declare LED 1 as output
  digitalWrite(ledPin, LOW);

  pinMode(ledPin2, OUTPUT);       // declare LED 2 as output
  digitalWrite(ledPin2, LOW);

  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED)
  {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");

  // Start the server
  server.begin();
  Serial.println("Server started");

  // Print the IP address
  Serial.print("Use this URL to connect: ");
  Serial.print("http://");
  Serial.print(WiFi.localIP());
  Serial.println("/");

  ThingSpeak.begin(client);
}

void loop()
{
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    return;
  }

  // Wait until the client sends some data
  Serial.println("new client");
  while (!client.available()) {
    delay(1);
  }

  // Read the first line of the request
  String request = client.readStringUntil('\r');
  Serial.println(request);
  client.flush();

  // Match the request

  // Servo motor for door
  int value1 = 0;

  if (request.indexOf("/Req=0") != -1)
  {
    servo.write(2);       //Moving servo to 0 degree
    value1 = 0;
  }

  if (request.indexOf("/Req=90") != -1)
  {
    servo.write(104);     //Moving servo to 90 degree
    value1 = 90;
  }

  // LED 1, LED 2 and PIR sensor
  long state = digitalRead(PIR);
  int value2 = LOW;

  if (request.indexOf("/LED=ON") != -1)
  {
    digitalWrite(ledPin, HIGH);
    value2 = HIGH;
  }

  if (request.indexOf("/MOT=ON") != -1)
  {

    if (state == HIGH)
    {
      digitalWrite (ledPin, HIGH);
      Serial.println("Motion detected!");
      delay(500);
    }

    else
    {
      digitalWrite (ledPin, LOW);
      Serial.println("Motion absent!");
      delay(500);
    }
  }

  if (request.indexOf("/LED=OFF") != -1)
  {
    digitalWrite(ledPin, LOW);
    value2 = LOW;
  }

  int value3 = LOW;
  if (request.indexOf("/LED2=ON") != -1)
  {
    digitalWrite(ledPin2, HIGH);
    value3 = HIGH;
  }

  if (request.indexOf("/LED2=OFF") != -1)
  {
    digitalWrite(ledPin2, LOW);
    value3 = LOW;
  }
  // Set ledPin according to the request
  // digitalWrite(ledPin, value);


  // LM35 and Fan
  float temperature = analogRead(LM35);
  temperature = (temperature * (3.3 / 1023));
  float celsius = temperature * 100.0;
  Serial.print("Temperature = ");
  Serial.print(celsius);
  Serial.println("C  ");
  val = celsius;

  ThingSpeak.writeField(myChannelNumber, 1, val, myWriteAPIKey); //Update in ThingSpeak

  if (request.indexOf("/FAN=ON") != -1)
  {
    if (celsius < 24)
    {
      analogWrite(FAN, 0);
      Serial.println("Fan Off");
      delay(500);
    }

    else if (celsius < 34)
    {
      analogWrite(FAN, 410);
      Serial.println("Fan Speed : Slow");
      delay(500);
    }

    else if (celsius < 38)
    {
      analogWrite(FAN, 754);
      Serial.println("Fan Speed : Medium");
      delay(500);
    }

    else if (celsius < 42)
    {
      analogWrite(FAN, 1023);
      Serial.println("Fan Speed : Fast");
      delay(500);
    }
  }

  // Fan Speed
  if (request.indexOf("/FAN=OFF") != -1)
  {
    analogWrite(FAN, 0);
    Serial.println("Fan Off");
    delay(500);
  }

  if (request.indexOf("/FAN=ON1") != -1)
  {
    analogWrite(FAN, 410);
    Serial.println("Fan Speed : Slow");
    delay(500);
  }

  if (request.indexOf("/FAN=ON2") != -1)
  {
    analogWrite(FAN, 754);
    Serial.println("Fan Speed : Medium");
    delay(500);
  }

  if (request.indexOf("/FAN=ON3") != -1)
  {
    analogWrite(FAN, 1023);
    Serial.println("Fan Speed : Fast");
    delay(500);
  }

  if (request.indexOf("/TEMP") != -1)
  {
    Serial.print(val);
    delay(500);
  }

  // Return the response
  client.println("HTTP/1.1 200 OK");
  client.println("Content-Type: text/html");
  client.println("Refresh: 5"); //Refresh every 5 seconds
  client.println(""); //  do not forget this one
  client.println("<!DOCTYPE HTML>");
  client.println("<html>");

  client.println("<h1 align=center>ISAT Server Control</h1><br><br>");

  client.print("<p style='text-align: center;'><span style='color: #0000ff;'><strong style='font-size: large;'>Temperature (*C)= ");
  client.println(celsius);

  client.println("<br><br>");

  client.print("Front Door");
  client.println("<br>");
  client.println("<a href=\"/Req=0\"\"><button>Close</button></a>");
  client.println("<a href=\"/Req=90\"\"><button>Open</button></a>");

  client.println("<br><br>");

  client.print("Living Room Light");
  client.println("<br>");
  client.println("<a href=\"/LED=OFF\"\"><button>Off </button></a>");
  client.println("<a href=\"/LED=ON\"\"><button>On </button></a>");
  client.println("<a href=\"/MOT=ON\"\"><button>MOn </button></a>");

  client.println("<br><br>");

  client.print("Bathroom Light");
  client.println("<br>");
  client.println("<a href=\"/LED2=OFF\"\"><button>Off </button></a>");
  client.println("<a href=\"/LED2=ON\"\"><button>On </button></a>");

  client.println("<br><br>");

  client.print("Living Room Fan");
  client.println("<br>");
  client.println("<a href=\"/FAN=OFF\"\"><button>Off </button></a>");
  client.println("<a href=\"/FAN=ON\"\"><button>On </button></a>");

  client.println("<br>");

  client.print("Living Room Fan Speed");
  client.println("<br>");
  client.println("<a href=\"/FAN=ON1\"\"><button>1 </button></a>");
  client.println("<a href=\"/FAN=ON2\"\"><button>2 </button></a>");
  client.println("<a href=\"/FAN=ON3\"\"><button>3 </button></a><br />");


  client.println("</html>");

  delay(100);
  Serial.println("Client disonnected");
  Serial.println("");

}
